from loguru import logger
import pandas as pd


from cleancourt.check_spaces import *
from cleancourt.clean_data import *
from cleancourt.cosine_similarity import *
from cleancourt.integrate_output import *
from cleancourt.parse_names import *
from cleancourt.separate_parent_company import separate_company_names


def full_clean_names(messy_names, separate_management=False, min_similarity_score=0.80):
    
    '''This is an abstracted function that calls the clean names function in CC in the order that they are intended to be called.'''


    if not separate_management:
        df = pd.DataFrame()

        df["clean_names_1"] = [clean_data(x) for x in tqdm(messy_names)]

        # Parse the people names and format
        logger.info("PARSING PEOPLE NAMES")
        duplicates_dict_people = format_people_names(df["clean_names_1"])

        # Merge names into df
        logger.info("MERGING PEOPLE NAMES")
        df["formatted_people_names"] = df.apply(
            lambda x: map_values(x.clean_names_1, duplicates_dict_people), axis=1
        )

        # Parse company names
        logger.info("PARSING COMPANY NAMES")
        duplicates_dict_company = link_company_names(
            df["clean_names_1"], min_similarity_score
        )

        # Merge company names into df
        logger.info("MERGING COMPANY NAMES")
        df["formatted_company_names"] = df.apply(
            lambda x: map_values(x.clean_names_1, duplicates_dict_company), axis=1
        )

        # Create column of all company and people names
        logger.info("ADDING ALL NAMES TO DF")
        df["clean_names_2"] = df.apply(
            lambda x: combine_columns(
                x.clean_names_1, x.formatted_people_names, x.formatted_company_names
            ),
            axis=1,
        )

        # Check for erroneous spaces that may prevent names from being matched
        logger.info("CHECKING SPACES")
        duplicates_dict_spaces = check_spaces(df["clean_names_2"])

        # add values returned from no spaces to df
        logger.info("ADDING NO SPACES NAMES TO DF")
        df["clean_names_3"] = df.apply(
            lambda x: map_values(x.clean_names_2, duplicates_dict_spaces), axis=1
        )

        # Combine all cleaned names columns into one column for use
        df["clean_name_final"] = df.apply(
            lambda x: combine_columns_no_space(x.clean_names_2, x.clean_names_3), axis=1
        )

        return df["clean_name_final"]
    else:
        df = pd.DataFrame()

        df["clean_names_1"] = [clean_data(x) for x in tqdm(messy_names)]

        logger.info("SEPARATING MANAGING NAMES")
        df["nameA"], df["nameB"] = separate_company_names(df["clean_names_1"])

        # Parse the people names and format
        logger.info("PARSING PEOPLE NAMES")
        duplicates_dict_peopleA = format_people_names(df["nameA"])
        duplicates_dict_peopleB = format_people_names(df["nameB"])

        # Merge names into df
        logger.info("MERGING PEOPLE NAMES")
        df["formatted_people_namesA"] = df.apply(
            lambda x: map_values(x.nameA, duplicates_dict_peopleA), axis=1
        )
        df["formatted_people_namesB"] = df.apply(
            lambda x: map_values(x.nameB, duplicates_dict_peopleB), axis=1
        )

        # Parse company names
        logger.info("PARSING COMPANY NAMES")
        duplicates_dict_companyA = link_company_names(df["nameA"], min_similarity_score)
        duplicates_dict_companyB = link_company_names(df["nameB"], min_similarity_score)

        # Merge company names into df
        logger.info("MERGING COMPANY NAMES")
        df["formatted_company_namesA"] = df.apply(
            lambda x: map_values(x.nameA, duplicates_dict_companyA), axis=1
        )
        df["formatted_company_namesB"] = df.apply(
            lambda x: map_values(x.nameB, duplicates_dict_companyB), axis=1
        )

        # Create column of all company and people names
        logger.info("ADDING ALL NAMES TO DF")
        df["clean_names_2a"] = df.apply(
            lambda x: combine_columns(
                x.nameA, x.formatted_people_namesA, x.formatted_company_namesA
            ),
            axis=1,
        )
        df["clean_names_2b"] = df.apply(
            lambda x: combine_columns(
                x.nameB, x.formatted_people_namesB, x.formatted_company_namesB
            ),
            axis=1,
        )

        # Check for erroneous spaces that may prevent names from being matched
        logger.info("CHECKING SPACES")
        duplicates_dict_spacesA = check_spaces(df["clean_names_2a"])
        duplicates_dict_spacesB = check_spaces(df["clean_names_2b"])

        # add values returned from no spaces to df
        logger.info("ADDING NO SPACES NAMES TO DF")
        df["clean_names_3a"] = df.apply(
            lambda x: map_values(x.clean_names_2a, duplicates_dict_spacesA), axis=1
        )
        df["clean_names_3b"] = df.apply(
            lambda x: map_values(x.clean_names_2b, duplicates_dict_spacesB), axis=1
        )

        # Combine all cleaned names columns into one column for use
        df["clean_name_finalA"] = df.apply(
            lambda x: combine_columns_no_space(x.clean_names_2a, x.clean_names_3a),
            axis=1,
        )
        df["clean_name_finalB"] = df.apply(
            lambda x: combine_columns_no_space(x.clean_names_2b, x.clean_names_3b),
            axis=1,
        )

        return (df["clean_name_finalA"], df["clean_name_finalB"])


# Same as the full clean names function, only this function separates the company names by DBA/TA names
# In future iterations, this method will replace full_clean_names
def full_clean_names1(messy_names):
    logger.error(
        "full_clean_names1 deprecated in cleancourt 0.6.0 use full_clean_names with separate_management=True"
    )
    quit()



from loguru import logger
import re
from tqdm import tqdm


def check_spaces(df):
    """takes in a list or df of one column and
    returns a dictionary of all names in the
    dataset that are found to be identical to at
    least one other name when all spaces are removed"""

    # Group data by exact matches and count the number of values
    logger.info("GROUPING DATA")
    grouped_df = (
        df.value_counts().rename_axis("first_plaintiff").reset_index(name="counts")
    )

    # Remove spaces and create new column
    grouped_df["no_space"] = grouped_df.apply(
        lambda x: re.sub(" +", "", x.first_plaintiff), axis=1
    )

    # See if no-space value is duplicate
    grouped_df["is_dup"] = grouped_df.duplicated(subset="no_space", keep=False)
    grouped_df = grouped_df[grouped_df.is_dup].reset_index()

    # sort dataframe for binary searching
    sorted_grouped_df = grouped_df.sort_values(by=["no_space"])
    duplicates_dict = {}

    # Get list of unique no_space names
    names = list(dict.fromkeys(grouped_df.no_space))

    # Iterate over unique no space names
    for name in tqdm(names):

        # binary search based on unique no space name
        indLeft = sorted_grouped_df["no_space"].searchsorted(name, "left")
        indRight = sorted_grouped_df["no_space"].searchsorted(name, "right")
        temp_df = sorted_grouped_df[indLeft:indRight]

        # Sort temp df created by counts, most frequently appearing term will come first
        temp_df = temp_df.sort_values(by=["counts"], ascending=False).reset_index()

        # Add the names into a dict, with the value that appears the most in the dataset as the version others are linked to
        for x in temp_df.first_plaintiff:

            duplicates_dict[x] = temp_df.first_plaintiff[0]

    return duplicates_dict



import re
from tqdm import tqdm


def remove_company_connector(string):
    original_string = string

    string = re.sub(
        "(.+) as a? ?management agent for (.{4,})", "\\2 /-/ \\1", string
    )  # Remove any appareance of as management agent for
    string = re.sub("(.+) successor in interest to (.{4,})", "\\2 /-/ \\1", string)
    string = re.sub("(.+) [as ]?assignee of (.{4,})", "\\2 /-/ \\1", string)
    string = re.sub("(.+) [as ]?managing agent for (.{4,})", "\\2 /-/ \\1", string)
    string = re.sub("(.+) [as ]?m/a for (.{4,})", "\\2 /-/ \\1", string)
    string = re.sub("(.{4,}) by (.+) as managing agent$", "\\1 /-/ \\2", string)
    string = re.sub(
        "(.+) a[^A-z]?s[^A-z]?o[^A-z]? (.{4,})", "\\2 /-/ \\1", string
    )  # Remove any appearance of a.s.o. and what follows
    string = re.sub(
        "(.+) d[^A-z]?b[^A-z]?a[^A-z]? (.{4,})", "\\2 /-/ \\1", string
    )  # Remove any appearance of d.b.a and what follows
    string = re.sub("(.+) as agent for (.{4,})", "\\2 /-/ \\1", string)
    string = re.sub("(.+) as agent (.{4,})", "\\2 /-/ \\1", string)
    string = re.sub("(.+) agent for (.{4,})", "\\2 /-/ \\1", string)
    string = re.sub("(.+) by its agent (.{4,})", "\\1 /-/ \\2", string)
    string = re.sub("(.+) by agent (.{4,})", "\\1 /-/ \\2", string)
    string = re.sub("(.+) as mg agent for (.{4,})", "\\2 /-/ \\1", string)
    string = re.sub("(.+) mg agt (.{4,})", "\\2 /-/ \\1", string)
    string = re.sub("(.+) as managing for (.{4,})", "\\2 /-/ \\1", string)
    string = re.sub("(.+) asset\.? management for (.{4,})", "\\2 /-/ \\1", string)
    string = re.sub("(.+) doing business as (.{4,})", "\\2 /-/ \\1", string)
    string = re.sub("(.+) owner of (.{4,})", "\\2 /-/ \\1", string)
    string = re.sub("(.{4,}) trustee of (.+)", "\\1 /-/ \\2", string)

    if (re.search("^owner /-/", string)):
        return original_string
    
    return string


def separate_company_names(names):
    plaintiffs = []
    plaintiff_parents = []

    for name in tqdm(names):
        split_names = re.split(" /-/ ", remove_company_connector(name))

        plaintiff_parents.append(split_names[1]) if (
            len(split_names) == 2
        ) else plaintiff_parents.append("")
        plaintiffs.append(split_names[0])

    return (plaintiffs, plaintiff_parents)



import ftfy
import re


def clean_data(string):
    """Takes in a string and cleans it using regex and other standard functions"""

    if not isinstance(string, str):
        string = ""

    string = ftfy.fix_text(
        string
    )  # Fix text to ensure any non-ascii characters appear properly
    string = string.encode(
        "ascii", errors="ignore"
    ).decode()  # Remove these non-ascii characters
    string = string.lower()  # Convert all characters to lowercase

    chars_to_remove = [
        ")",
        "(",
        "[",
        "]",
        "{",
        "}",
        "'",
        "#",
        ";",
    ]  # remove these characters
    rx = "[" + re.escape("".join(chars_to_remove)) + "]"
    string = re.sub(rx, "", string)

    # Replace special characters and symbols
    string = string.replace("&", " and ")
    string = string.replace("@", " at ")
    #string = re.sub(' {2}', ' ', string)

    string = string.replace(
        "h/w", ""
    )  # Sometimes appears next to party names that are husband (h) and wife (w)
    string = string.replace("-", " ")
    string = string.replace("_", " ")

    string = re.sub(
        " +", " ", string
    ).strip()  # Remove multiple spaces and spaces at the end of a word

    string = re.sub("^the ", "", string)  # Remove 'the' at the beginning of party names
    string = re.sub(
        " i+ ", " ", string
    )  # Remove one or more 'i' characters that appear in isolation (denotes numbers)

    string = re.sub(" c/o.*", "", string)  # Remove care of

    string = re.sub(" corporation", " corp", string)  # shorten corporation to corp
    string = re.sub(" company", " co", string)  # shorten company to co
    string = re.sub(" maa?nage?ment", " mgmt", string)  # etc
    string = re.sub(" incorporated", " inc", string)
    string = re.sub(" apartment", " apt", string)
    string = re.sub(" square(\s|$|,)", " sq\\1", string)
    string = re.sub("department", "dept", string)

    string = re.sub("fed[^\s]{0,4} cred[^\s]{0,2} un[^\s]{0,4}", "fcu", string)
    string = re.sub("credit union", "fcu", string)

    string = re.sub("home ?owners association", "hoa", string)
    string = re.sub("home ?owners assoc[^\s]{0,3}", "hoa", string)
    string = re.sub("hoa,? inc", "hoa", string)

    string = re.sub("national association", "na", string)

    string = re.sub("housing authy", "housing authority", string) # this replacement is being placed before any of the other housing authority replacements because "authy" appears in both PHA and RHA names
    string = re.sub("housing authority, inc", "housing authority", string)

    string = re.sub("redevelopment and housing authority", "rha", string)
    string = re.sub("redevelopment and housing auth[^\s]?", "rha", string)
    string = re.sub("redevelopment and housing", "rha", string)
    string = re.sub("redevelopment housing authority", "rha", string)

    string = re.sub("public housing authority", "pha", string)
    string = re.sub("metropolitan housing authority", "pha", string)
    string = re.sub("housing authority", "pha", string)
    string = re.sub("housing and redevelopment authority", "pha", string)

    

    string = re.sub(" et\.?\s?al\.?$", "", string)
    string = re.sub(
        "p\.?c\.?$", "", string
    )  # Remove P.C. (Personal Corporation) abbreviation on the end of a name

    string = re.sub("\.", "", string)  # Remove any periods

    return string



from loguru import logger
import probablepeople as pp
from tqdm import tqdm
import re

from cleancourt.filter_types import filter_people


def format_people_names(df):

    """Take in a list or df of one column and return a dictionary with each name in First Last format"""

    # Group data by exact matches and count the number of values
    logger.info("GROUPING DATA")
    grouped_df = (
        df.value_counts().rename_axis("first_plaintiff").reset_index(name="counts")
    )

    # Grab the names of each plaintiff and place into a list
    # Note: Order of this list does matter.
    # Names are prioritized based on the number of exact matches in the document

    names = grouped_df.first_plaintiff

    names_dict = {}

    # Iterate over names

    for name in tqdm(names):

        # Check if name is a person
        is_person = filter_people(name)

        # If is a person, grab name components from parsed names and place in a common format
        if is_person:
            val = pp.tag(name)
            try:
                firstI = val[0]["FirstInitial"]
            except KeyError:
                firstI = ""

            try:
                prefOther = val[0]["PrefixOther"]
            except KeyError:
                prefOther = ""


            try: # Update 0.7.8 first names are sometimes parsed as SuffixOther in probablepeople. this attempts to correct for that error
                sufOther = val[0]["SuffixOther"]
                if sufOther == 'dds' or sufOther == 'md' or sufOther == 'dmd' or sufOther == 'dpn':
                    sufOtherEnd = sufOther
                    sufOther = ""
                else:
                    sufOtherEnd = ""

            except KeyError:
                sufOther = ""
                sufOtherEnd = ""


            try:
                first = val[0]["GivenName"]
            except KeyError:
                first = ""

            try:
                middle = val[0]["MiddleName"]
            except KeyError:
                middle = ""

            try:
                middleI = val[0]["MiddleInitial"]
            except KeyError:
                middleI = ""

            try:
                last = val[0]["Surname"]
            except KeyError:
                last = ""

            try:
                lastI = val[0]["LastInitial"]
            except KeyError:
                lastI = ""

            try:
                suffix = val[0]["SuffixGenerational"]
            except KeyError:
                suffix = ""

            full_name = ( # Add all of the parsed names together in a standard format
                firstI
                + " "
                + prefOther
                + " "
                + first
                + " "
                + sufOther
                + " "
                + middle
                + " "
                + middleI
                + " "
                + last
                + " " 
                + lastI
                + " "
                + suffix
                + " " 
                + sufOtherEnd
            )

            names_dict[name] = re.sub(" +", " ", full_name.strip()) # Strip any extra white spaces

    return names_dict



from setuptools import setup, find_packages
import os

here = os.path.abspath(os.path.dirname(__file__))


def get_file_text(file_name):
    with open(os.path.join(here, file_name)) as in_file:
        return in_file.read()


setup(
    name="cleancourt",
    version="0.7.12",
    description="a library for cleaning court docket entries",
    author="Logan Pratico",
    author_email="praticol@lsc.gov",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    long_description=get_file_text("README.md"),
    long_description_content_type="text/markdown",
    keywords="name standardization, plaintiff names, court data, name cleaning",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    python_requires=">=3.9, <4",
    install_requires=[
        "scikit-learn",
        "loguru",
        "probablepeople",
        "rapidfuzz",
        "tqdm",
        "pandas",
        "sparse-dot-topn",
        "ftfy",
        "scipy",
        "numpy",
    ],
    project_urls={},
)
